@@ -207,7 +207,7 @@ function virtualenvwrapper_run_hook {
     # Python 3 interpreter decide that its "prefix" is the virtualenv
     # if we happen to be inside the virtualenv when we start.
     ( \
-        cd "$WORKON_HOME" &&
+        virtualenvwrapper_cd "$WORKON_HOME" &&
         "$VIRTUALENVWRAPPER_PYTHON" -m 'virtualenvwrapper.hook_loader' \
             $HOOK_VERBOSE_OPTION --script "$hook_script" "$@" \
     )
@@ -288,6 +288,8 @@ function virtualenvwrapper_initialize {
         export VIRTUALENVWRAPPER_HOOK_DIR="$WORKON_HOME"
     fi
 
+    mkdir -p "$VIRTUALENVWRAPPER_HOOK_DIR"
+
     virtualenvwrapper_run_hook "initialize"
 
     virtualenvwrapper_setup_tab_completion
@@ -383,6 +385,7 @@ function mkvirtualenv {
     typeset requirements
     typeset packages
     typeset interpreter
+    typeset project
 
     in_args=( "$@" )
 
@@ -400,8 +403,14 @@ function mkvirtualenv {
         # echo "arg $i : $a"
         case "$a" in
             -a)
-                i=$(( $i + 1 ));
-                project="${in_args[$i]}";;
+                i=$(( $i + 1 ))
+                project="${in_args[$i]}"
+                if [ ! -d "$project" ]
+                then
+                    echo "Cannot associate project with $project, it is not a directory" 1>&2
+                    return 1
+                fi
+                project="$(virtualenvwrapper_absolutepath ${project})";;
             -h|--help)
                 virtualenvwrapper_mkvirtualenv_help $a;
                 return;;
@@ -462,6 +471,8 @@ function mkvirtualenv {
     if [ ! -z "$project" ]
     then
         setvirtualenvproject "$WORKON_HOME/$envname" "$project"
+        RC=$?
+        [ $RC -ne 0 ] && return $RC
     fi
 
     # Now activate the new environment
@@ -534,7 +545,7 @@ function virtualenvwrapper_show_workon_options {
     # 4. Format the output to show one name on a line.
     # 5. Eliminate any lines with * on them because that means there 
     #    were no envs.
-    (cd "$WORKON_HOME" && echo */$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate) 2>/dev/null \
+    (virtualenvwrapper_cd "$WORKON_HOME" && echo */$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate) 2>/dev/null \
         | command \sed "s|/$VIRTUALENVWRAPPER_ENV_BIN_DIR/activate||g" \
         | command \fmt -w 1 \
         | (unset GREP_OPTIONS; command \egrep -v '^\*$') 2>/dev/null
@@ -967,21 +978,46 @@ function setvirtualenvproject {
     if [ -z "$prj" ]
     then
         prj="$(pwd)"
+    else
+        prj=$(virtualenvwrapper_absolutepath "${prj}")
     fi
+
+    # If what we were given isn't a directory, see if it is under
+    # $WORKON_HOME.
+    if [ ! -d "$venv" ]
+    then
+        venv="$WORKON_HOME/$venv"
+    fi
+    if [ ! -d "$venv" ]
+    then
+        echo "No virtualenv $(basename $venv)" 1>&2
+        return 1
+    fi
+
+    # Make sure we have a valid project setting
+    if [ ! -d "$prj" ]
+    then
+        echo "Cannot associate virtualenv with \"$prj\", it is not a directory" 1>&2
+        return 1
+    fi
+
     echo "Setting project for $(basename $venv) to $prj"
     echo "$prj" > "$venv/$VIRTUALENVWRAPPER_PROJECT_FILENAME"
 }
 
 # Show help for mkproject
 function virtualenvwrapper_mkproject_help {
-    echo "Usage: mkproject [-t template] [virtualenv options] project_name"
-    echo ""
+    echo "Usage: mkproject [-f|--force] [-t template] [virtualenv options] project_name"
+    echo
+    echo "-f, --force    Create the virtualenv even if the project directory"
+    echo "               already exists"
+    echo
     echo "Multiple templates may be selected.  They are applied in the order"
     echo "specified on the command line."
-    echo;
+    echo
     echo "mkvirtualenv help:"
     echo
-    mkvirtualenv -h;
+    mkvirtualenv -h
     echo
     echo "Available project templates:"
     echo
@@ -996,9 +1032,11 @@ function mkproject {
     typeset tst
     typeset a
     typeset t
+    typeset force
     typeset templates
 
     in_args=( "$@" )
+    force=0
 
     if [ -n "$ZSH_VERSION" ]
     then
@@ -1015,6 +1053,8 @@ function mkproject {
             -h|--help)
                 virtualenvwrapper_mkproject_help;
                 return;;
+            -f|--force)
+                force=1;;
             -t)
                 i=$(( $i + 1 ));
                 templates="$templates ${in_args[$i]}";;
@@ -1038,7 +1078,7 @@ function mkproject {
     eval "typeset envname=\$$#"
     virtualenvwrapper_verify_project_home || return 1
 
-    if [ -d "$PROJECT_HOME/$envname" ]
+    if [ -d "$PROJECT_HOME/$envname" -a $force -eq 0 ]
     then
         echo "Project $envname already exists." >&2
         return 1
@@ -1046,7 +1086,7 @@ function mkproject {
 
     mkvirtualenv "$@" || return 1
 
-    cd "$PROJECT_HOME"
+    virtualenvwrapper_cd "$PROJECT_HOME"
 
     virtualenvwrapper_run_hook "project.pre_mkproject" $envname
 
@@ -1054,7 +1094,7 @@ function mkproject {
     mkdir -p "$PROJECT_HOME/$envname"
     setvirtualenvproject "$VIRTUAL_ENV" "$PROJECT_HOME/$envname"
 
-    cd "$PROJECT_HOME/$envname"
+    virtualenvwrapper_cd "$PROJECT_HOME/$envname"
 
     for t in $templates
     do
@@ -1078,7 +1118,7 @@ function cdproject {
         typeset project_dir="$(cat "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_PROJECT_FILENAME")"
         if [ ! -z "$project_dir" ]
         then
-            cd "$project_dir"
+            virtualenvwrapper_cd "$project_dir"
         else
             echo "Project directory $project_dir does not exist" 1>&2
             return 1
@@ -1106,6 +1146,7 @@ function mktmpenv {
         # This python does not support uuid
         tmpenvname=$("$VIRTUALENVWRAPPER_PYTHON" -c 'import random,sys; sys.stdout.write(hex(random.getrandbits(64))[2:-1]+"\n")' 2>/dev/null)
     fi
+    tmpenvname="tmp-$tmpenvname"
 
     # Create the environment
     mkvirtualenv "$@" "$tmpenvname"
@@ -1167,8 +1208,8 @@ function allvirtualenv {
         # Activate the environment, but not with workon
         # because we don't want to trigger any hooks.
         (source "$WORKON_HOME/$d/bin/activate";
-            cd "$VIRTUAL_ENV";
-            $@)
+            virtualenvwrapper_cd "$VIRTUAL_ENV";
+            "$@")
         echo
     done
 }
