@@ -19,12 +19,6 @@ GLOBAL_HOOKS = [
      "and before it is activated."),
     ("postmkproject",
      "This hook is run after a new project is activated."),
-
-    # rmproject
-    ("prermproject",
-     "This hook is run before a project is deleted."),
-    ("postrmproject",
-     "This hook is run after a project is deleted."),
 ]
 
 
@@ -58,5 +52,6 @@ def post_activate_source(args):
 # Change to the project directory
 #
 [ -f "$VIRTUAL_ENV/$VIRTUALENVWRAPPER_PROJECT_FILENAME" ] && \
-    cd "$(cat \"$VIRTUAL_ENV/$VIRTUALENVWRAPPER_PROJECT_FILENAME\")"
+    virtualenvwrapper_cd \
+        "$(cat \"$VIRTUAL_ENV/$VIRTUALENVWRAPPER_PROJECT_FILENAME\")"
 """
